@@ -17,7 +17,7 @@
  *
  *
  *
- *  $Id: utilities.h,v 1.6 2003/02/16 15:06:09 adnans Exp $
+ *  $Id: utilities.h 1030 2004-02-15 18:58:30Z adnans $
  * 
  */ 
 
@@ -36,10 +36,13 @@ extern "C" {
 // Used by scopes and things, so use C-style linkage
 void dosleep(unsigned int);
 void parse_file_uri_free(char *);
+void parse_percent_free(char *);
 char *get_homedir(void);
 char *get_prefsdir(void);
 char *parse_file_uri(const char *);
+char *parse_percent(const char *);
 int is_playlist(const char *);
+int is_uri(const char *);
 
 #ifdef __cplusplus
 }
