@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  *
- *  $Id: CorePlayer.cpp,v 1.73 2003/11/10 19:57:45 adnans Exp $
+ *  $Id: CorePlayer.cpp 1114 2007-05-19 10:22:25Z dominique_libre $
  *
 */ 
 
@@ -29,6 +29,7 @@
 
 #include "AlsaPlayer.h"
 #include "CorePlayer.h"
+#include <cstdlib>
 #include <cstdio>
 #include <cstring>
 #include <cassert>
@@ -206,6 +207,7 @@ CorePlayer::CorePlayer(AlsaNode *the_node)
 	pthread_mutex_init(&counter_mutex, NULL); 
 	pthread_mutex_init(&player_mutex, NULL);
 	pthread_mutex_init(&notifier_mutex, NULL);
+	pthread_cond_init(&producer_ready, NULL);
 	
 	producer_thread = 0; 
 	total_frames = 0;
@@ -651,11 +653,8 @@ void CorePlayer::Stop()
 	}
 	producing = false;
 	streaming = false;
-
-	/* We don't know locked it or not */
-	pthread_mutex_trylock (&counter_mutex);
-	pthread_mutex_unlock (&counter_mutex);
-
+	//alsaplayer_error("Signalling 6...");
+	pthread_cond_signal(&producer_ready);
 #ifdef DEBUG	
 	alsaplayer_error("Waiting for producer_thread to finish...");
 #endif
@@ -663,7 +662,7 @@ void CorePlayer::Stop()
 	    if (pthread_join(producer_thread, NULL)) {
 		    alsaplayer_error("producer_thread not running...");
 	    } 
-	}	    
+	}
 	pthread_mutex_destroy(&counter_mutex);
 	
 	pthread_mutex_init(&counter_mutex, NULL);
@@ -918,7 +917,8 @@ int CorePlayer::SetDirection(int direction)
 		new_frame_number = new_write_buf->start;
 		write_buf_changed = 1;
 		read_direction = direction;
-		pthread_mutex_unlock(&counter_mutex);		
+		//alsaplayer_error("Signalling...1");
+		pthread_cond_signal(&producer_ready);		
 	}
 	return 0;	
 	
@@ -1013,7 +1013,8 @@ int CorePlayer::Read32(void *data, int samples)
 		new_write_buf->start = jump_point;
 		new_frame_number = jump_point;
 		write_buf_changed = 1;
-		pthread_mutex_unlock(&counter_mutex);
+		//alsaplayer_error("Signalling 2...");
+		pthread_cond_signal(&producer_ready);
 	}
 #ifdef EMBEDDED
 	// provide a fast path implementation for the common case (no pitch)
@@ -1040,7 +1041,8 @@ int CorePlayer::Read32(void *data, int samples)
 
 			// get next block
 			read_buf = read_buf->next;
-			pthread_mutex_unlock(&counter_mutex);
+			//alsaplayer_error("Signalling 3...");
+			pthread_cond_signal(&producer_ready);
 			read_buf->buf->SetReadDirection(DIR_FORWARD);
 			read_buf->buf->ResetRead();
 			in_buf = (int *)read_buf->buf->buffer_data;
@@ -1073,7 +1075,8 @@ int CorePlayer::Read32(void *data, int samples)
 					read_buf->start);
 				}
 				read_buf = read_buf->next;
-				pthread_mutex_unlock(&counter_mutex);
+				//alsaplayer_error("Signalling 4...");
+				pthread_cond_signal(&producer_ready);
 				read_buf->buf->SetReadDirection(DIR_FORWARD);
 				read_buf->buf->ResetRead();
 				in_buf = (int *)read_buf->buf->buffer_data;
@@ -1100,7 +1103,8 @@ int CorePlayer::Read32(void *data, int samples)
 					return samples;
 				}
 				read_buf = read_buf->prev;
-				pthread_mutex_unlock(&counter_mutex);
+				//alsaplayer_error("Signalling 5...");
+				pthread_cond_signal(&producer_ready);
 				read_buf->buf->SetReadDirection(DIR_BACK);
 				read_buf->buf->ResetRead();
 				int buf_size = read_buf->buf->write_index;
@@ -1149,7 +1153,7 @@ int CorePlayer::pcm_worker(sample_buf *dest, int start)
 #ifdef DEBUG
 			alsaplayer_error("frames read = %d", frames_read);
 #endif
-			return frames_read;
+                        count = 0;
 		} else {
 			bytes_written += plugin->frame_size(the_object); // OPTIMIZE!
 			frames_read++;
@@ -1217,6 +1221,10 @@ void CorePlayer::producer_func(void *data)
 		} else {
 			//alsaplayer_error("producer: waiting for free buffer");
 			pthread_mutex_lock(&obj->counter_mutex);
+			//alsaplayer_error("Waiting for a signal...");
+			pthread_cond_wait(&obj->producer_ready, &obj->counter_mutex);
+			//alsaplayer_error("Got signalled...");
+			pthread_mutex_unlock(&obj->counter_mutex);
 			//alsaplayer_error("producer: unblocked");
 		}	
 	}
@@ -1306,7 +1314,7 @@ bool CorePlayer::IsPaused()
 
 void CorePlayer::Pause()
 {
-	save_speed = GetSpeed ();
+	save_speed = (GetSpeed () * ((float) output_rate / (float) input_rate));
 	SetSpeed (0.0);
 }
 
@@ -1316,5 +1324,5 @@ void CorePlayer::UnPause()
 	if (save_speed)
 	    SetSpeed (save_speed);
 	else
-	    SetSpeed (1.0);
+	SetSpeed (1.0);
 }
