@@ -1,5 +1,5 @@
 /*  Main.cpp - main() function and other utils
- *  Copyright (C) 1998-2003 Andy Lo A Foe <andy@alsaplayer.org>
+ *  Copyright (C) 1998-2004 Andy Lo A Foe <andy@alsaplayer.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  *
- *  $Id: Main.cpp,v 1.109 2003/09/18 17:31:23 sjah Exp $
+ *  $Id: Main.cpp 1060 2007-02-01 13:52:04Z dominique_libre $
  *
  *  VIM: set ts=8
  *
@@ -186,7 +186,7 @@ extern bool reverb_func(void *arg, void *data, int size);
 
 static char *copyright_string =
     "AlsaPlayer " VERSION
-    "\n(C) 1999-2003 Andy Lo A Foe <andy@alsaplayer.org> and others.";
+    "\n(C) 1999-2004 Andy Lo A Foe <andy@alsaplayer.org> and others.";
 
 static void list_available_plugins(const char *plugindir)
 {
@@ -916,13 +916,17 @@ int main(int argc, char **argv)
 			goto _fatal_err;
 		}
 	} else {
-		if (!(interface_plugin_info = load_interface(prefs_get_string
-						(ap_prefs, "main", "default_interface", "gtk")))) {
+		char *interface = prefs_get_string
+			(ap_prefs, "main", "default_interface", "gtk");
+		// if we're trying to use the gtk interface, but we have no
+		// $DISPLAY, use the text interface instead
+		if (strcmp (interface, "gtk") == 0 && !getenv("DISPLAY"))
+			interface = "text";
+		if (!(interface_plugin_info = load_interface(interface))) {
 			if (!(interface_plugin_info = load_interface(prefs_get_string
 						 (ap_prefs, "main", "fallback_interface", "text")))) {
 				alsaplayer_error("Failed to load text interface. This is bad (%s,%s,%s)",
-					 prefs_get_string(ap_prefs, "main", "default_interface", "gtk"),
-					 prefs_get_string(ap_prefs, "main", "default_interface", "gtk"),
+					 interface, interface,
 					global_pluginroot);
 				goto _fatal_err;
 			}
