@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  *
- *  $Id: Playlist.cpp,v 1.89 2003/11/02 18:30:46 adnans Exp $
+ *  $Id: Playlist.cpp 1046 2005-01-30 21:28:00Z adnans $
  *
 */
 
@@ -255,7 +255,7 @@ void playlist_looper(void *data)
 	if(!pl) return;
 
 	while(pl->active) {
-		if (!pl->Paused()) {
+		if (!pl->IsPaused()) {
 			if (!(coreplayer = (CorePlayer *)(pl->coreplayer)))
 				return;
 			
@@ -277,7 +277,7 @@ void playlist_looper(void *data)
 				int totaltime = coreplayer->GetCurrentTime(nr_frames);
 				
 				float frametime = (float)totaltime  / (float)nr_frames;
-				float xstart = 40; // 0.40 seconds
+				float xstart = 300; // 3.0 seconds
 				float xframe = xstart / frametime;
 				//alsaplayer_error("xframe = %.2f", xframe);
 				if ((coreplayer->GetFrames() - coreplayer->GetPosition()) < (int)xframe) {
@@ -444,6 +444,7 @@ Playlist::~Playlist() {
 		delete player2;
 
 	Lock();
+	Unlock();
 	pthread_mutex_destroy(&playlist_mutex);
 }
 
@@ -807,6 +808,7 @@ Playlist::Load(std::string const &uri, unsigned position, bool force)
 
 	// Read the file
 	char path[READBUFSIZE + 1];
+	memset(path, 0,READBUFSIZE);
 	std::vector<std::string> newfiles;
 
 	// Give up if too many failures (so we don't wait for almost ever if
@@ -1034,6 +1036,17 @@ bool Playlist::Eof()
 	return false;
 }
 
+
+PlayItem *Playlist::GetItem(unsigned item)
+{
+        if(item < 1) item = 1;
+	if(item > queue.size()) {
+		item = queue.size();
+	}
+	return &queue[item-1];
+}
+
+
 // Add a path to list, recursing through (to a maximum of depth subdirectories)
 static void additems(std::vector<std::string> *items, std::string path, int depth) {
 	if(depth < 0) return;
