@@ -8,7 +8,9 @@
 #include <arpa/inet.h>
 #include <sys/stat.h>		/* for fd operations		*/
 #include <fcntl.h>		/* for fd operations, too	*/
+#define __USE_XOPEN /* needed for swab */
 #include <unistd.h>		/* for close()			*/
+#undef __USE_XOPEN
 #include <errno.h>		/* errno..     		        */
 
 #endif /* _cdda_h */
