@@ -1,6 +1,6 @@
 /*
  * libmad - MPEG audio decoder library
- * Copyright (C) 2000-2001 Robert Leslie
+ * Copyright (C) 2000-2004 Underbit Technologies, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: huffman.c,v 1.3 2002/04/30 12:11:28 adnans Exp $
+ * $Id: huffman.c 1042 2004-05-17 22:42:18Z fbaumgart $
  */
 
 # ifdef HAVE_CONFIG_H
@@ -34,13 +34,19 @@
  * These tables support decoding up to 4 Huffman code bits at a time.
  */
 
-# if defined(__GNUC__)
-#  define PTR(offs, bits)	{ ptr:   { 0, bits, offs       } }
-#  define V(v, w, x, y, hlen)	{ value: { 1, hlen, v, w, x, y } }
+# if defined(__GNUC__) ||  \
+    (defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901)
+#  define PTR(offs, bits)	{ .ptr   = { 0, bits, offs       } }
+#  define V(v, w, x, y, hlen)	{ .value = { 1, hlen, v, w, x, y } }
 # else
 #  define PTR(offs, bits)	{ { 0, bits, offs } }
-#  define V(v, w, x, y, hlen)	{ { 1, hlen, (v << 0) | (w << 1) |  \
-                                             (x << 2) | (y << 3) } }
+#  if defined(WORDS_BIGENDIAN)
+#   define V(v, w, x, y, hlen)	{ { 1, hlen, (v << 11) | (w << 10) |  \
+                                             (x <<  9) | (y <<  8) } }
+#  else
+#   define V(v, w, x, y, hlen)	{ { 1, hlen, (v <<  0) | (w <<  1) |  \
+                                             (x <<  2) | (y <<  3) } }
+#  endif
 # endif
 
 static
@@ -106,12 +112,17 @@ union huffquad const hufftabB[] = {
 # undef V
 # undef PTR
 
-# if defined(__GNUC__)
-#  define PTR(offs, bits)	{ ptr:   { 0, bits, offs } }
-#  define V(x, y, hlen)		{ value: { 1, hlen, x, y } }
+# if defined(__GNUC__) ||  \
+    (defined(__STDC_VERSION__) && __STDC_VERSION__ >= 199901)
+#  define PTR(offs, bits)	{ .ptr   = { 0, bits, offs } }
+#  define V(x, y, hlen)		{ .value = { 1, hlen, x, y } }
 # else
 #  define PTR(offs, bits)	{ { 0, bits, offs } }
-#  define V(x, y, hlen)		{ { 1, hlen, (x << 0) | (y << 4) } }
+#  if defined(WORDS_BIGENDIAN)
+#   define V(x, y, hlen)	{ { 1, hlen, (x << 8) | (y << 4) } }
+#  else
+#   define V(x, y, hlen)	{ { 1, hlen, (x << 0) | (y << 4) } }
+#  endif
 # endif
 
 static
