@@ -18,7 +18,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  *
- *  $Id: mad_engine.c,v 1.69 2003/05/16 19:19:11 adnans Exp $
+ *  $Id: mad_engine.c 1031 2004-02-15 19:01:07Z adnans $
  */ 
 
 #ifdef HAVE_CONFIG_H
@@ -894,7 +894,7 @@ static int mad_open(input_object *obj, const char *path)
 	memset(data, 0, sizeof(struct mad_local_data));
 
 	if ((data->mad_fd = reader_open(path, &reader_status, obj)) == NULL) {
-		fprintf(stderr, "mad_open() failed\n");
+		fprintf(stderr, "mad_open(obj, %s) failed\n", path);
 		free(obj->local_data);
 		obj->local_data = NULL;
 		return 0;
