@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  *
- *  $Id: message.c,v 1.34 2003/03/27 07:58:27 adnans Exp $
+ *  $Id: message.c 1038 2004-02-25 20:31:00Z adnans $
  *
  */ 
 #include "message.h"
@@ -503,21 +503,49 @@ int ap_cmd_set_int(int session, int32_t cmd, int val)
 }
 
 
-
-/**
- * Convenience function for commands that return a single int
- *
- * @param int session Session to contact
- * @param int32_t cmd Command to execute
- * @param int *val Variable where the result will be copied to
- * @return true if command succeeded, false otherwise
- */
-int ap_cmd_get_int(int session, int32_t cmd, int *val)
+/* Convenience function for commands that take a single float */
+int ap_cmd_set_float(int session, int32_t cmd, float val)
 {
 	int fd;
 	ap_message_t *msg, *reply;
 	int32_t *result;
 	
+	fd = ap_connect_session(session);
+	if (fd < 0)
+		return 0;
+	msg = ap_message_new();
+	msg->header.cmd = cmd;
+	ap_message_add_float(msg, "float", val);
+
+	ap_message_send(fd, msg);
+	ap_message_delete(msg);
+	msg = NULL;
+
+	reply = ap_message_receive(fd);
+	close(fd);
+
+	if ((result = ap_message_find_int32(reply, "ack"))) {
+		ap_message_delete(reply);
+		return 1;
+	}
+	ap_message_delete(reply);
+	return 0;
+}
+
+/**
+ * Convenience function for commands that return a single float
+ *
+ * @param int session Session to contact
+ * @param int32_t cmd Command to execute
+ * @param float *val Variable where the result will be copied to
+ * @return true if command succeeded, false otherwise
+ */
+int ap_cmd_get_float(int session, int32_t cmd, float *val)
+{
+	int fd;
+	ap_message_t *msg, *reply;
+	float *res;
+	
 	fd = ap_connect_session(session);
 	if (fd < 0)
 		return 0;
@@ -531,8 +559,45 @@ int ap_cmd_get_int(int session, int32_t cmd, int *val)
 	reply = ap_message_receive(fd);
 	close(fd);
 
-	if ((result = ap_message_find_int32(reply, "int"))) {
-		*val = *result;
+	if ((res = ap_message_find_float(reply, "float"))) {
+		*val = *res;
+		ap_message_delete(reply);
+		return 1;
+	}
+	ap_message_delete(reply);
+	return 0;
+}
+
+
+/**
+ * Convenience function for commands that return a single int
+ *
+ * @param int session Session to contact
+ * @param int32_t cmd Command to execute
+ * @param int *val Variable where the result will be copied to
+ * @return true if command succeeded, false otherwise
+ */
+int ap_cmd_get_int(int session, int32_t cmd, int *val)
+{
+	int fd;
+	ap_message_t *msg, *reply;
+	int32_t *res;
+	
+	fd = ap_connect_session(session);
+	if (fd < 0)
+		return 0;
+	msg = ap_message_new();
+	msg->header.cmd = cmd;
+
+	ap_message_send(fd, msg);
+	ap_message_delete(msg);
+	msg = NULL;
+
+	reply = ap_message_receive(fd);
+	close(fd);
+
+	if ((res = ap_message_find_int32(reply, "int"))) {
+		*val = *res;
 		ap_message_delete(reply);
 		return 1;
 	}
@@ -566,14 +631,14 @@ int ap_is_paused(int session , int *paused)
 	 return (ap_cmd_get_int(session, AP_IS_PAUSED, paused));
 }
 
-int ap_set_volume(int session, int volume)
+int ap_set_volume(int session, float volume)
 {
-	return (ap_cmd_set_int(session, AP_SET_VOLUME, volume));
+	return (ap_cmd_set_float(session, AP_SET_VOLUME, volume));
 }
 
-int ap_set_pan(int session, int pan)
+int ap_set_pan(int session, float pan)
 {
-	return (ap_cmd_set_int(session, AP_SET_PAN, pan));
+	return (ap_cmd_set_float(session, AP_SET_PAN, pan));
 }
 
 int ap_set_position(int session, int pos)
@@ -597,14 +662,14 @@ int ap_get_tracks(int session, int *val)
 }
 
 
-int ap_get_volume(int session, int *volume)
+int ap_get_volume(int session, float *volume)
 {
-	return (ap_cmd_get_int(session, AP_GET_VOLUME, volume));
+	return (ap_cmd_get_float(session, AP_GET_VOLUME, volume));
 }
 
-int ap_get_pan(int session, int *pan)
+int ap_get_pan(int session, float *pan)
 {
-	return (ap_cmd_get_int(session, AP_GET_PAN, pan));
+	return (ap_cmd_get_float(session, AP_GET_PAN, pan));
 }
 
 int ap_get_length(int session, int *length)
@@ -977,7 +1042,152 @@ int ap_quit(int session)
 	return (ap_do_command_only(session, AP_QUIT));
 }
 
+int ap_get_playlist_position(int session, int *pos)
+{
+	return (ap_cmd_get_int(session, AP_GET_PLAYLIST_POSITION, pos));
+}
 
+int ap_get_string_set_int(int session, int32_t cmd, char* str, int maxlen, int val)
+{
+	int fd, *ack;
+	ap_message_t *msg, *reply;
+	char *result;
+
+	fd = ap_connect_session(session);
+	if (fd < 0)
+		return 0;
+	msg = ap_message_new();
+	msg->header.cmd = cmd;
+	ap_message_add_int32(msg, "int", val);
+
+	ap_message_send(fd, msg);
+	ap_message_delete(msg);
+	msg = NULL;
+
+	reply = ap_message_receive(fd);
+	close(fd);
+
+	ack = ap_message_find_int32(reply, "ack");
+	if(*ack == 1) {
+	  if ((result = ap_message_find_string(reply, "string"))) {
+	    if (strlen(result) > (size_t)maxlen) {
+	      strncpy(str, result, maxlen-1);
+	      str[maxlen] = 0;
+	    } else 	
+	      strcpy(str, result);
+	    ap_message_delete(reply);
+	    return 1;
+	  }
+	}
+	ap_message_delete(reply);
+	return 0;
+}
+
+int ap_get_file_path_for_track(int session, char* path, int track)
+{
+        return (ap_get_string_set_int(session, AP_GET_FILE_PATH_FOR_TRACK, path, AP_FILE_PATH_MAX, track));
+}
+
+int ap_set_string_set_int(int session, int32_t cmd, const char* string, int val)
+{
+	int fd;
+	int32_t *result;
+	ap_message_t *msg, *reply;
+
+	fd = ap_connect_session(session);
+	if (fd < 0)
+		return 0;
+	msg = ap_message_new();
+	msg->header.cmd = cmd;
+
+	ap_message_add_string(msg, "string", string);
+	ap_message_add_int32(msg, "int", val);
+
+	ap_message_send(fd, msg);
+	ap_message_delete(msg);
+	msg = NULL;
+
+	reply = ap_message_receive(fd);
+	close(fd);
+
+	result = ap_message_find_int32(reply, "ack");
+	if(*result) {
+	  ap_message_delete(reply);
+	  return 1;
+	}
+	ap_message_delete(reply);
+	return 0;
+}        
+
+int ap_get_playlist(int session, int *argc, char ***the_list) {
+	int fd;
+	char **list;
+	ap_message_t *msg, *reply;
+	int *result, nritems, c, t;
+	char strnum[64];
+	char *res;
+	
+	fd = ap_connect_session(session);
+	if (fd < 0)
+		return 0;
+	msg = ap_message_new();
+	msg->header.cmd = AP_GET_PLAYLIST;
+	
+	ap_message_send(fd, msg);
+	ap_message_delete(msg);
+	msg = NULL;
+
+	reply = ap_message_receive(fd);
+	close(fd);
+	
+	if ((result = ap_message_find_int32(reply, "items"))) {
+		nritems = *result;
+		if (nritems > 0) {
+			*argc = nritems;
+		} else {
+			*argc = 0;
+			ap_message_delete(reply);
+			return 1;
+		}	
+		list = (char **)malloc(sizeof(char *) * nritems);
+		if (!list) {
+			ap_message_delete(reply);
+			return 0;
+		}	
+		for (c=0; c < nritems; c++) {
+			sprintf(strnum, "%d", c+1);
+			res = ap_message_find_string(reply, strnum);
+			if (res) {
+				list[c] = strdup(res);
+				if (!list[c]) {
+					printf("error!");
+					ap_message_delete(reply);
+					// XXX memory leak XXX
+					return 0;
+				}
+				//printf("Got string (%s): %s\n", strnum, list[c]);
+			}
+		}
+		*the_list = list;
+		ap_message_delete(reply);
+		return 1;
+	}
+	ap_message_delete(reply);
+	return 0;
+
+}
+
+int ap_insert(int session, const char* path, int val) {
+    return ap_set_string_set_int(session, AP_INSERT, path, val);
+}
+
+int ap_remove(int session, int pos) {
+    return ap_cmd_set_int(session, AP_REMOVE, pos);
+}
+
+int ap_set_current(int session, int pos) {
+    return ap_cmd_set_int(session, AP_SET_CURRENT, pos);
+}
 
 #ifdef __cplusplus
 } /* extern "C" */
