@@ -56,7 +56,7 @@ static int jack_prepare(void *arg);
 static void jack_shutdown(void *arg);
 static void jack_restarter(void *arg);
 
-#define SAMPLE_MAX_16BIT  32767.0f
+#define SAMPLE_MAX_16BIT  32768.0f
 
 void sample_move_dS_s16 (sample_t *dst, char *src,
 		unsigned long nsamples, unsigned long src_skip) 
@@ -157,16 +157,10 @@ int jack_prepare(void *arg)
 			if (jack_connect (client, jack_port_name(my_output_port1), dest_port1)) {
 				alsaplayer_error("cannot connect output port 1 (%s)",
 						dest_port1);
-				free(mix_buffer);
-				mix_buffer = NULL;
-				return -1;
 			}               
 			if (jack_connect (client, jack_port_name(my_output_port2), dest_port2)) {
 				alsaplayer_error("cannot connect output port 2 (%s)",
 						dest_port2);
-				free(mix_buffer);
-				mix_buffer = NULL;
-				return -1;
 			}
 		}
 #ifdef TEST_MASTER		
